{% extends '!layout.html' %}

{%- block sidebarsearch %}
{{ super() }}
    <div id="searchbox">
	<h3>Google Search</h3>
    <form method="get" action="http://www.google.com/search">

    <table border="0" cellpadding="0">
    <tr><td>
    <input type="text"   name="q" size="20"
    maxlength="255" value="" />
    <input type="submit" value="Go" /></td></tr>
    <tr><td align="center" style="font-size:75%; color: #FFFFFF;">
    <input type="checkbox"  name="sitesearch"
    value="http://github.enthought.com/mayavi/mayavi/" checked /> only search Mayavi documentation<br />
    </td></tr></table>

    </form>
    </div>
    <hr/>
    <div id="searchbox">
	<a href="https://mayavi.readthedocs.io/en/latest/misc.html#citing-mayavi"><h3>Citing Mayavi</h3></a>

	<p style="font-size:85%; color: #FFFFFF;">
    	If you publish articles using Mayavi, please 
	<strong><a href="misc.html#citing-mayavi">cite
	Mayavi</a></strong>. We need
	these citations to justify time and resources on the software.
	</p>
    </div>
{% endblock %}

{# the palette moved to _static/mayavi.css, which conf.py loads via
   html_css_files so the landing page can share it #}
