/* Mayavi's palette on top of the classic theme.  Loaded by the docs through
   html_css_files, and by the site landing page (docs/site/index.html) so the
   two cannot drift apart. */

body {
  /* Very bottom copyright, etc. info */
  background: #4d598d;
}

.documentwrapper {
  /* Blank area on left bar */
  background: #24326e;
}

.related {
  /* Long horizontal nav bars */
  background: #3c4b8a!important;
}

.sphinxsidebarwrapper {
  /* Filled part of left sidebar */
  background: #24326e;
}

a {
  color: #253370;
}

.sphinxsidebarwrapper a {
  color: #f2f2f2!important;
}
